import lal
import lalsimulation

import numpy as np
from scipy import linalg
from pylab import figure, show, rand
from matplotlib.patches import Ellipse
from math import pow, atan, degrees
import matplotlib.pyplot as plt

def mceta_from_m1m2(m1,m2):
    M = m1+m2
    mu = m1*m2/M
    eta = mu/M
    Mc = pow(mu*mu*mu*M*M,1./5.)
    
    return Mc, eta

# set the frequency range and resolution
f_low = 10.
f_high = 2048.
df = 0.1

# set the masses and hence chirptimes
m1 = 1.4
m2 = 1.4
Mc,eta = mceta_from_m1m2(m1,m2)
# chi = 0 because we don't care about spin.  don't change this!
chi = 0
# compute chirp times
chirp_times = lalsimulation.SimInspiralTaylorF2RedSpinChirpTimesFromMchirpEtaChi(Mc, eta, chi, f_low)
                                                                   
# Sampled PSD.
S = lal.CreateREAL8Vector(int(f_high // df))
# SimNoisePSDaLIGOZeroDetHighPower is the high power zero detuning advanced LIGO noise curve
S.data = [lalsimulation.SimNoisePSDaLIGOZeroDetHighPower(i * df)
    for i in range(len(S.data))]

# Allocate noise moments.
moments = [lal.CreateREAL8Vector(int((f_high - f_low) // df)) for _ in range(29)]

# Compute noise moments.
lalsimulation.SimInspiralTaylorF2RedSpinComputeNoiseMoments(
    *(moments + [S, f_low, df]))

# Compute Fisher information matrix. Note factor of 2:
# The function SimInspiralTaylorF2RedSpinFisherMatrixChirpTimes
# returns the Fisher matrix for an SNR of 1/sqrt(2). The factor
# of 2 makes this the Fisher matrix at unit SNR.
F = lalsimulation.SimInspiralTaylorF2RedSpinFisherMatrixChirpTimes(
    *(chirp_times + [f_low, df] + moments)).data * 2


# some explanation:  the fisher matrix returned has the components listed below (copied from the c code)
# where:
# theta0 = 2 pi fLow tau0 
# theta3 = -2 pi fLow tau3 
# theta3s = 2 pi fLow tau3s ***this should be zero since we are setting chi=0
# t0, phi0 are as expected
#   
#  gsl_matrix_set (g, 0,0, g_theta0theta0/hSqr);
#  gsl_matrix_set (g, 0,1, g_theta0theta3/hSqr);
#  gsl_matrix_set (g, 0,2, g_theta0theta3s/hSqr);
#  gsl_matrix_set (g, 0,3, g_theta0t0/hSqr);
#  gsl_matrix_set (g, 0,4, g_theta0phi0/hSqr);
#
#  gsl_matrix_set (g, 1,0, gsl_matrix_get(g, 0,1));
#  gsl_matrix_set (g, 1,1, g_theta3theta3/hSqr);
#  gsl_matrix_set (g, 1,2, g_theta3theta3s/hSqr);
#  gsl_matrix_set (g, 1,3, g_theta3t0/hSqr);
#  gsl_matrix_set (g, 1,4, g_theta3phi0/hSqr);
#
#  gsl_matrix_set (g, 2,0, gsl_matrix_get(g, 0,2));
#  gsl_matrix_set (g, 2,1, gsl_matrix_get(g, 1,2));
#  gsl_matrix_set (g, 2,2, g_theta3stheta3s/hSqr);
#  gsl_matrix_set (g, 2,3, g_theta3st0/hSqr);
#  gsl_matrix_set (g, 2,4, g_theta3sphi0/hSqr);
#
#  gsl_matrix_set (g, 3,0, gsl_matrix_get(g, 0,3));
#  gsl_matrix_set (g, 3,1, gsl_matrix_get(g, 1,3));
#  gsl_matrix_set (g, 3,2, gsl_matrix_get(g, 2,3));
#  gsl_matrix_set (g, 3,3, g_t0t0/hSqr);
#  gsl_matrix_set (g, 3,4, g_t0phi0/hSqr);
#
#  gsl_matrix_set (g, 4,0, gsl_matrix_get(g, 0,4));
#  gsl_matrix_set (g, 4,1, gsl_matrix_get(g, 1,4));
#  gsl_matrix_set (g, 4,2, gsl_matrix_get(g, 2,4));
#  gsl_matrix_set (g, 4,3, gsl_matrix_get(g, 3,4));
#  gsl_matrix_set (g, 4,4, g_phi0phi0/hSqr);



#theta0 theta3
fsub_theta0theta3 = np.array([[F[0,0], F[0,1]],[F[1,0], F[1,1]]])
cov_theta0theta3 = linalg.inv(fsub_theta0theta3)
evals_theta0theta3 = linalg.eigvals(cov_theta0theta3)
angle_theta0theta3 = degrees(.5*atan((2*cov_theta0theta3[0,1])/(cov_theta0theta3[0,0]-cov_theta0theta3[1,1])))

#theta0 theta3s
fsub_theta0theta3s = np.array([[F[0,0], F[0,2]],[F[2,0], F[2,2]]])
cov_theta0theta3s = linalg.inv(fsub_theta0theta3s)
evals_theta0theta3s = linalg.eigvals(cov_theta0theta3s)
angle_theta0theta3s = degrees(.5*atan((2*cov_theta0theta3s[0,1])/(cov_theta0theta3s[0,0]-cov_theta0theta3s[1,1])))

#theta0 t0
fsub_theta0t0 = np.array([[F[0,0], F[0,3]],[F[3,0], F[3,3]]])
cov_theta0t0 = linalg.inv(fsub_theta0t0)
evals_theta0t0 = linalg.eigvals(cov_theta0t0)
angle_theta0t0 = degrees(.5*atan((2*cov_theta0t0[0,1])/(cov_theta0t0[0,0]-cov_theta0t0[1,1])))

#theta0 phi0
fsub_theta0phi0 = np.array([[F[0,0], F[0,4]],[F[4,0], F[4,4]]])
cov_theta0phi0 = linalg.inv(fsub_theta0phi0)
evals_theta0phi0 = linalg.eigvals(cov_theta0phi0)
angle_theta0phi0 = degrees(.5*atan((2*cov_theta0phi0[0,1])/(cov_theta0phi0[0,0]-cov_theta0phi0[1,1])))

#theta3 theta3s
fsub_theta3theta3s = np.array([[F[1,1], F[1,2]],[F[2,1], F[2,2]]])
cov_theta3theta3s = linalg.inv(fsub_theta3theta3s)
evals_theta3theta3s = linalg.eigvals(cov_theta3theta3s)
angle_theta3theta3s = degrees(.5*atan((2*cov_theta3theta3s[0,1])/(cov_theta3theta3s[0,0]-cov_theta3theta3s[1,1])))

#theta3 t0
fsub_theta3t0 = np.array([[F[1,1], F[1,3]],[F[3,1], F[3,3]]])
cov_theta3t0 = linalg.inv(fsub_theta3t0)
evals_theta3t0 = linalg.eigvals(cov_theta3t0)
angle_theta3t0 = degrees(.5*atan((2*cov_theta3t0[0,1])/(cov_theta3t0[0,0]-cov_theta3t0[1,1])))

#theta3 phi0
fsub_theta3phi0 = np.array([[F[1,1], F[1,4]],[F[4,1], F[4,4]]])
cov_theta3phi0 = linalg.inv(fsub_theta3phi0)
evals_theta3phi0 = linalg.eigvals(cov_theta3phi0)
angle_theta3phi0 = degrees(.5*atan((2*cov_theta3phi0[0,1])/(cov_theta3phi0[0,0]-cov_theta3phi0[1,1])))

#theta3s t0
fsub_theta3st0 = np.array([[F[2,2], F[2,3]],[F[3,2], F[3,3]]])
cov_theta3st0 = linalg.inv(fsub_theta3st0)
evals_theta3st0 = linalg.eigvals(cov_theta3st0)
angle_theta3st0 = degrees(.5*atan((2*cov_theta3st0[0,1])/(cov_theta3st0[0,0]-cov_theta3st0[1,1])))

#theta3s phi0
fsub_theta3sphi0 = np.array([[F[2,2], F[2,4]],[F[4,2], F[4,4]]])
cov_theta3sphi0 = linalg.inv(fsub_theta3sphi0)
evals_theta3sphi0 = linalg.eigvals(cov_theta3sphi0)
angle_theta3sphi0 = degrees(.5*atan((2*cov_theta3sphi0[0,1])/(cov_theta3sphi0[0,0]-cov_theta3sphi0[1,1])))

#t0 phi0
fsub_t0phi0 = np.array([[F[3,3], F[3,4]],[F[4,3], F[4,4]]])
cov_t0phi0 = linalg.inv(fsub_t0phi0)
evals_t0phi0 = linalg.eigvals(cov_t0phi0)
angle_t0phi0 = degrees(.5*atan((2*cov_t0phi0[0,1])/(cov_t0phi0[0,0]-cov_t0phi0[1,1])))


fig = plt.figure(figsize=(8,12))

#theta0 theta3
ax = fig.add_subplot(521, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $\theta_3$ and $\theta_0$')
ax.set_xlabel(r'$\sigma_{\theta_0}$ (rad)')
ax.set_ylabel(r'$\sigma_{\theta_3}$ (rad)')
ellipse_theta0theta3 = Ellipse((0,0), width=2*pow(evals_theta0theta3[0],.5), height=2*pow(evals_theta0theta3[1],.5), angle=angle_theta0theta3)
ax.add_artist(ellipse_theta0theta3)
ellipse_theta0theta3.set_clip_box(ax.bbox)
ellipse_theta0theta3.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)

#theta0 theta3s
ax = fig.add_subplot(522, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $\theta_{3s}$ and $\theta_0$')
ax.set_xlabel(r'$\sigma_{\theta_0}$ (rad)')
ax.set_ylabel(r'$\sigma_{\theta_{3s}}$ (rad)')
ellipse_theta0theta3s = Ellipse((0,0), width=2*pow(evals_theta0theta3s[0],.5), height=2*pow(evals_theta0theta3s[1],.5), angle=angle_theta0theta3s)
ax.add_artist(ellipse_theta0theta3s)
ellipse_theta0theta3s.set_clip_box(ax.bbox)
ellipse_theta0theta3s.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)

#theta0 t0
ax = fig.add_subplot(523, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $t_0$ and $\theta_0$')
ax.set_xlabel(r'$\sigma_{\theta_0}$ (rad)')
ax.set_ylabel(r'$\sigma_{t_0}$ (s)')
ellipse_theta0t0 = Ellipse((0,0), width=2*pow(evals_theta0t0[0],.5), height=2*pow(evals_theta0t0[1],.5), angle=angle_theta0t0)
ax.add_artist(ellipse_theta0t0)
ellipse_theta0t0.set_clip_box(ax.bbox)
ellipse_theta0t0.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)

#theta0 phi0
ax = fig.add_subplot(524, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $\phi_0$ and $\theta_0$')
ax.set_xlabel(r'$\sigma_{\theta_0}$ (rad)')
ax.set_ylabel(r'$\sigma_{\phi_0}$ (rad)')
ellipse_theta0phi0 = Ellipse((0,0), width=2*pow(evals_theta0phi0[0],.5), height=2*pow(evals_theta0phi0[1],.5), angle=angle_theta0phi0)
ax.add_artist(ellipse_theta0phi0)
ellipse_theta0phi0.set_clip_box(ax.bbox)
ellipse_theta0phi0.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)

#theta3 theta3s
ax = fig.add_subplot(525, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $\theta_{3s}$ and $\theta_3$')
ax.set_xlabel(r'$\sigma_{\theta_3}$ (rad)')
ax.set_ylabel(r'$\sigma_{\theta_{3s}}$ (rad)')
ellipse_theta3theta3s = Ellipse((0,0), width=2*pow(evals_theta3theta3s[0],.5), height=2*pow(evals_theta3theta3s[1],.5), angle=angle_theta3theta3s)
ax.add_artist(ellipse_theta3theta3s)
ellipse_theta3theta3s.set_clip_box(ax.bbox)
ellipse_theta3theta3s.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)

#theta3 t0
ax = fig.add_subplot(526, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $t_0$ and $\theta_3$')
ax.set_xlabel(r'$\sigma_{\theta_3}$ (rad)')
ax.set_ylabel(r'$\sigma_{t_0}$ (s)')
ellipse_theta3t0 = Ellipse((0,0), width=2*pow(evals_theta3t0[0],.5), height=2*pow(evals_theta3t0[1],.5), angle=angle_theta3t0)
ax.add_artist(ellipse_theta3t0)
ellipse_theta3t0.set_clip_box(ax.bbox)
ellipse_theta3t0.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)

#theta3 phi0
ax = fig.add_subplot(527, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $\phi_0$ and $\theta_3$')
ax.set_xlabel(r'$\sigma_{\theta_3}$ (rad)')
ax.set_ylabel(r'$\sigma_{\phi_0}$ (rad)')
ellipse_theta3phi0 = Ellipse((0,0), width=2*pow(evals_theta3phi0[0],.5), height=2*pow(evals_theta3phi0[1],.5), angle=angle_theta3phi0)
ax.add_artist(ellipse_theta3phi0)
ellipse_theta3phi0.set_clip_box(ax.bbox)
ellipse_theta3phi0.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)

#theta3s t0
ax = fig.add_subplot(528, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $t_0$ and $\theta_{3s}$')
ax.set_xlabel(r'$\sigma_{\theta_{3s}}$ (rad)')
ax.set_ylabel(r'$\sigma_{t_0}$ (s)')
ellipse_theta3st0 = Ellipse((0,0), width=2*pow(evals_theta3st0[0],.5), height=2*pow(evals_theta3st0[1],.5), angle=angle_theta3st0)
ax.add_artist(ellipse_theta3st0)
ellipse_theta3st0.set_clip_box(ax.bbox)
ellipse_theta3st0.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)

#theta3s phi0
ax = fig.add_subplot(529, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $\phi_0$ and $\theta_{3s}$')
ax.set_xlabel(r'$\sigma_{\theta_{3s}}$ (rad)')
ax.set_ylabel(r'$\sigma_{\phi_0}$ (rad)')
ellipse_theta3sphi0 = Ellipse((0,0), width=2*pow(evals_theta3sphi0[0],.5), height=2*pow(evals_theta3sphi0[1],.5), angle=angle_theta3sphi0)
ax.add_artist(ellipse_theta3sphi0)
ellipse_theta3sphi0.set_clip_box(ax.bbox)
ellipse_theta3sphi0.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)

#t0 phi0
ax = fig.add_subplot(5,2,10, aspect='auto')
fig.subplots_adjust(top=0.85)
ax.set_title(r'Uncertainty in $\phi_0$ and $t_0$')
ax.set_xlabel(r'$\sigma_{t_0}$ (s)')
ax.set_ylabel(r'$\sigma_{\phi_0}$ (rad)')
ellipse_t0phi0 = Ellipse((0,0), width=2*pow(evals_t0phi0[0],.5), height=2*pow(evals_t0phi0[1],.5), angle=angle_t0phi0)
ax.add_artist(ellipse_t0phi0)
ellipse_t0phi0.set_clip_box(ax.bbox)
ellipse_t0phi0.set_facecolor('white')
ax.set_xlim(-60,60)
ax.set_ylim(-35,35)
max_yticks = 5
yloc = plt.MaxNLocator(max_yticks)
ax.yaxis.set_major_locator(yloc)


plt.show()